@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -42,6 +42,8 @@
 #include "glib-helper.h"
 #include "btio.h"
 #include "dbus-common.h"
+#include "adapter.h"
+#include "device.h"
 
 #include "error.h"
 #include "common.h"
@@ -59,6 +61,7 @@ struct network_peer {
 	bdaddr_t	src;
 	bdaddr_t	dst;
 	char		*path;		/* D-Bus path */
+	struct btd_device *device;
 	GSList		*connections;
 };
 
@@ -67,8 +70,9 @@ struct network_conn {
 	char		dev[16];	/* Interface name */
 	uint16_t	id;		/* Role: Service Class Identifier */
 	conn_state	state;
-	int		sk;
+	GIOChannel	*io;
 	guint		watch;		/* Disconnect watch */
+	guint		dc_id;
 	struct network_peer *peer;
 };
 
@@ -158,6 +162,8 @@ static gboolean bnep_watchdog_cb(GIOChannel *chan, GIOCondition cond,
 		emit_property_changed(connection, nc->peer->path,
 					NETWORK_PEER_INTERFACE, "UUID",
 					DBUS_TYPE_STRING, &property);
+		device_remove_disconnect_watch(nc->peer->device, nc->dc_id);
+		nc->dc_id = 0;
 		if (nc->watch) {
 			g_dbus_remove_watch(connection, nc->watch);
 			nc->watch = 0;
@@ -170,12 +176,53 @@ static gboolean bnep_watchdog_cb(GIOChannel *chan, GIOCondition cond,
 	nc->state = DISCONNECTED;
 	memset(nc->dev, 0, 16);
 	strncpy(nc->dev, prefix, sizeof(nc->dev) - 1);
-	g_io_channel_shutdown(chan, TRUE, NULL);
 
 	return FALSE;
 }
 
-static gboolean bnep_connect_cb(GIOChannel *chan, GIOCondition cond,
+static void cancel_connection(struct network_conn *nc, const char *err_msg)
+{
+	DBusMessage *reply;
+
+	if (nc->watch) {
+		g_dbus_remove_watch(connection, nc->watch);
+		nc->watch = 0;
+	}
+
+	if (nc->msg && err_msg) {
+		reply = connection_attempt_failed(nc->msg, err_msg);
+		g_dbus_send_message(connection, reply);
+	}
+
+	g_io_channel_shutdown(nc->io, TRUE, NULL);
+	g_io_channel_unref(nc->io);
+	nc->io = NULL;
+
+	nc->state = DISCONNECTED;
+}
+
+static void connection_destroy(DBusConnection *conn, void *user_data)
+{
+	struct network_conn *nc = user_data;
+
+	if (nc->state == CONNECTED) {
+		bnep_if_down(nc->dev);
+		bnep_kill_connection(&nc->peer->dst);
+	} else if (nc->io)
+		cancel_connection(nc, NULL);
+}
+
+static void disconnect_cb(struct btd_device *device, gboolean removal,
+				void *user_data)
+{
+	struct network_conn *nc = user_data;
+
+	info("Network: disconnect %s", nc->peer->path);
+
+	connection_destroy(NULL, user_data);
+}
+
+static gboolean bnep_setup_cb(GIOChannel *chan, GIOCondition cond,
 							gpointer data)
 {
 	struct network_conn *nc = data;
@@ -184,7 +231,6 @@ static gboolean bnep_connect_cb(GIOChannel *chan, GIOCondition cond,
 	char pkt[BNEP_MTU];
 	gsize r;
 	int sk;
-	DBusMessage *reply;
 	const char *pdev, *uuid;
 	gboolean connected;
 
@@ -263,20 +309,20 @@ static gboolean bnep_connect_cb(GIOChannel *chan, GIOCondition cond,
 				DBUS_TYPE_STRING, &uuid);
 
 	nc->state = CONNECTED;
+	nc->dc_id = device_add_disconnect_watch(nc->peer->device, disconnect_cb,
+						nc, NULL);
 
 	info("%s connected", nc->dev);
 	/* Start watchdog */
 	g_io_add_watch(chan, G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 			(GIOFunc) bnep_watchdog_cb, nc);
+	g_io_channel_unref(nc->io);
+	nc->io = NULL;
+
 	return FALSE;
 
 failed:
-	if (nc->state != DISCONNECTED) {
-		nc->state = DISCONNECTED;
-		reply = connection_attempt_failed(nc->msg, strerror(EIO));
-		g_dbus_send_message(connection, reply);
-		g_io_channel_shutdown(chan, TRUE, NULL);
-	}
+	cancel_connection(nc, "bnep setup failed");
 
 	return FALSE;
 }
@@ -287,8 +333,7 @@ static int bnep_connect(struct network_conn *nc)
 	struct __service_16 *s;
 	struct timeval timeo;
 	unsigned char pkt[BNEP_MTU];
-	GIOChannel *io;
-	int err = 0;
+	int fd;
 
 	/* Send request */
 	req = (void *) pkt;
@@ -302,28 +347,21 @@ static int bnep_connect(struct network_conn *nc)
 	memset(&timeo, 0, sizeof(timeo));
 	timeo.tv_sec = 30;
 
-	setsockopt(nc->sk, SOL_SOCKET, SO_RCVTIMEO, &timeo, sizeof(timeo));
+	fd = g_io_channel_unix_get_fd(nc->io);
+	setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &timeo, sizeof(timeo));
 
-	io = g_io_channel_unix_new(nc->sk);
-	g_io_channel_set_close_on_unref(io, FALSE);
+	if (send(fd, pkt, sizeof(*req) + sizeof(*s), 0) < 0)
+		return -errno;
 
-	if (send(nc->sk, pkt, sizeof(*req) + sizeof(*s), 0) < 0) {
-		err = -errno;
-		goto out;
-	}
+	g_io_add_watch(nc->io, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
+			(GIOFunc) bnep_setup_cb, nc);
 
-	g_io_add_watch(io, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
-			(GIOFunc) bnep_connect_cb, nc);
-
-out:
-	g_io_channel_unref(io);
-	return err;
+	return 0;
 }
 
 static void connect_cb(GIOChannel *chan, GError *err, gpointer data)
 {
 	struct network_conn *nc = data;
-	DBusMessage *reply;
 	const char *err_msg;
 	int perr;
 
@@ -333,41 +371,17 @@ static void connect_cb(GIOChannel *chan, GError *err, gpointer data)
 		goto failed;
 	}
 
-	nc->sk = g_io_channel_unix_get_fd(chan);
-
 	perr = bnep_connect(nc);
 	if (perr < 0) {
 		err_msg = strerror(-perr);
 		error("bnep connect(): %s (%d)", err_msg, -perr);
-		g_io_channel_shutdown(chan, TRUE, NULL);
-		g_io_channel_unref(chan);
 		goto failed;
 	}
 
 	return;
 
 failed:
-	nc->state = DISCONNECTED;
-	if (nc->watch) {
-		g_dbus_remove_watch(connection, nc->watch);
-		nc->watch = 0;
-	}
-
-	reply = connection_attempt_failed(nc->msg, err_msg);
-	g_dbus_send_message(connection, reply);
-}
-
-static void connection_destroy(DBusConnection *conn, void *user_data)
-{
-	struct network_conn *nc = user_data;
-
-	nc->watch = 0;
-
-	if (nc->state == CONNECTED) {
-		bnep_if_down(nc->dev);
-		bnep_kill_connection(&nc->peer->dst);
-	} else
-		close(nc->sk);
+	cancel_connection(nc, err_msg);
 }
 
 /* Connect and initiate BNEP session */
@@ -379,7 +393,6 @@ static DBusMessage *connection_connect(DBusConnection *conn,
 	const char *svc;
 	uint16_t id;
 	GError *err = NULL;
-	GIOChannel *io;
 
 	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &svc,
 						DBUS_TYPE_INVALID) == FALSE)
@@ -393,7 +406,7 @@ static DBusMessage *connection_connect(DBusConnection *conn,
 	if (nc->state != DISCONNECTED)
 		return already_connected(msg);
 
-	io = bt_io_connect(BT_IO_L2CAP, connect_cb, nc,
+	nc->io = bt_io_connect(BT_IO_L2CAP, connect_cb, nc,
 				NULL, &err,
 				BT_IO_OPT_SOURCE_BDADDR, &peer->src,
 				BT_IO_OPT_DEST_BDADDR, &peer->dst,
@@ -401,7 +414,7 @@ static DBusMessage *connection_connect(DBusConnection *conn,
 				BT_IO_OPT_OMTU, BNEP_MTU,
 				BT_IO_OPT_IMTU, BNEP_MTU,
 				BT_IO_OPT_INVALID);
-	if (!io) {
+	if (!nc->io) {
 		DBusMessage *reply;
 		error("%s", err->message);
 		reply = connection_attempt_failed(msg, err->message);
@@ -409,8 +422,6 @@ static DBusMessage *connection_connect(DBusConnection *conn,
 		return reply;
 	}
 
-	g_io_channel_unref(io);
-
 	nc->state = CONNECTING;
 	nc->msg = dbus_message_ref(msg);
 	nc->watch = g_dbus_add_disconnect_watch(conn,
@@ -431,12 +442,7 @@ static DBusMessage *connection_cancel(DBusConnection *conn,
 	if (!g_str_equal(owner, caller))
 		return not_permited(msg);
 
-	if (nc->watch) {
-		g_dbus_remove_watch(conn, nc->watch);
-		nc->watch = 0;
-	}
-
-	connection_destroy(conn, data);
+	connection_destroy(conn, nc);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -511,11 +517,10 @@ static DBusMessage *connection_get_properties(DBusConnection *conn,
 
 static void connection_free(struct network_conn *nc)
 {
-	if (nc->state == CONNECTED) {
-		bnep_if_down(nc->dev);
-		bnep_kill_connection(&nc->peer->dst);
-	} else if (nc->state == CONNECTING)
-		close(nc->sk);
+	if (nc->dc_id)
+		device_remove_disconnect_watch(nc->peer->device, nc->dc_id);
+
+	connection_destroy(connection, nc);
 
 	g_free(nc);
 	nc = NULL;
@@ -525,6 +530,7 @@ static void peer_free(struct network_peer *peer)
 {
 	g_slist_foreach(peer->connections, (GFunc) connection_free, NULL);
 	g_slist_free(peer->connections);
+	btd_device_unref(peer->device);
 	g_free(peer->path);
 	g_free(peer);
 }
@@ -533,7 +539,7 @@ static void path_unregister(void *data)
 {
 	struct network_peer *peer = data;
 
-	info("Unregistered interface %s on path %s",
+	debug("Unregistered interface %s on path %s",
 		NETWORK_PEER_INTERFACE, peer->path);
 
 	peers = g_slist_remove(peers, peer);
@@ -574,12 +580,14 @@ void connection_unregister(const char *path, uint16_t id)
 	g_dbus_unregister_interface(connection, path, NETWORK_PEER_INTERFACE);
 }
 
-static struct network_peer *create_peer(const char *path, bdaddr_t *src,
-				bdaddr_t *dst)
+static struct network_peer *create_peer(struct btd_device *device,
+					const char *path, bdaddr_t *src,
+					bdaddr_t *dst)
 {
 	struct network_peer *peer;
 
 	peer = g_new0(struct network_peer, 1);
+	peer->device = btd_device_ref(device);
 	peer->path = g_strdup(path);
 	bacpy(&peer->src, src);
 	bacpy(&peer->dst, dst);
@@ -595,14 +603,14 @@ static struct network_peer *create_peer(const char *path, bdaddr_t *src,
 		return NULL;
 	}
 
-	info("Registered interface %s on path %s",
+	debug("Registered interface %s on path %s",
 		NETWORK_PEER_INTERFACE, path);
 
 	return peer;
 }
 
-int connection_register(const char *path, bdaddr_t *src, bdaddr_t *dst,
-			uint16_t id)
+int connection_register(struct btd_device *device, const char *path,
+			bdaddr_t *src, bdaddr_t *dst, uint16_t id)
 {
 	struct network_peer *peer;
 	struct network_conn *nc;
@@ -612,7 +620,7 @@ int connection_register(const char *path, bdaddr_t *src, bdaddr_t *dst,
 
 	peer = find_peer(peers, path);
 	if (!peer) {
-		peer = create_peer(path, src, dst);
+		peer = create_peer(device, path, src, dst);
 		if (!peer)
 			return -1;
 		peers = g_slist_append(peers, peer);
