@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2009  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2010  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -102,13 +102,16 @@ static int conn_list(int s, int dev_id, long arg)
 
 	for (i = 0; i < cl->conn_num; i++, ci++) {
 		char addr[18];
+		char *str;
 		ba2str(&ci->bdaddr, addr);
+		str = hci_lmtostr(ci->link_mode);
 		printf("\t%s %s %s handle %d state %d lm %s\n",
 			ci->out ? "<" : ">", type2str(ci->type),
-			addr, ci->handle, ci->state,
-			hci_lmtostr(ci->link_mode));
+			addr, ci->handle, ci->state, str);
+		bt_free(str);
 	}
 
+	free(cl);
 	return 0;
 }
 
@@ -132,9 +135,12 @@ static int find_conn(int s, int dev_id, long arg)
 	}
 
 	for (i = 0; i < cl->conn_num; i++, ci++)
-		if (!bacmp((bdaddr_t *) arg, &ci->bdaddr))
+		if (!bacmp((bdaddr_t *) arg, &ci->bdaddr)) {
+			free(cl);
 			return 1;
+		}
 
+	free(cl);
 	return 0;
 }
 
@@ -503,7 +509,6 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 	uint8_t lap[3] = { 0x33, 0x8b, 0x9e };
 	int num_rsp, length, flags;
 	uint8_t cls[3], features[8];
-	uint16_t handle;
 	char addr[18], name[249], oui[9], *comp, *tmp;
 	struct hci_version version;
 	struct hci_dev_info di;
@@ -603,6 +608,8 @@ static void cmd_scan(int dev_id, int argc, char **argv)
 		printf("\n");
 
 	for (i = 0; i < num_rsp; i++) {
+		uint16_t handle = 0;
+
 		if (!refresh) {
 			memset(name, 0, sizeof(name));
 			tmp = get_device_name(&di.bdaddr, &(info+i)->bdaddr);
@@ -888,6 +895,7 @@ static void cmd_info(int dev_id, int argc, char **argv)
 			close(dd);
 			exit(1);
 		}
+		sleep(1);
 		cc = 1;
 	} else
 		handle = htobs(cr->conn_info->handle);
